
/*	Copyright (C) 2005, 2006 by Andy Grebe a.k.a. tecknophreak(tecknophreak@gmail.com)
 *
 *	This file is part of TouchMe Games.
 *
 *	TouchMe Games is free software; you can redistribute it and/or modify
 *	it under the terms of the GNU General Public License as published by
 *	the Free Software Foundation; either version 2 of the License, or
 *	(at your option) any later version.
 *
 *	TouchMe Games is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public License
 *	along with TouchMe Games; if not, write to the Free Software
 *	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

// support functions for DartCricket10

#include <dartcricket10.h>
#include <sstream>

DartCricket10::DartCricket10() : round(1), doubleBullOn(false), lastChanceDarts(false), gameState(PLAYING) {
	GAME_PTR::setScoreType(FLOAT);
	
	bgcolor.red = 0x4600;
	bgcolor.green = 0x8200;
	bgcolor.blue = 0xB400;
	for (int i = 0; i < 8; ++i) {
		darts[i] = 0;
		rounds[i] = 0;
		doubles[i] = 0;
		triples[i] = 0;
	}
	winner = -1;
	beginningScore = 0;
	didTon = false;
}

DartCricket10::~DartCricket10() {
}

std::string DartCricket10::name() const {
	return "dartcricket10";
}

std::string DartCricket10::imageName() const {
	return "dartcricket10.gif";
}
GdkColor *DartCricket10::backgroundColor() {
	return &bgcolor;
}

GtkWidget *DartCricket10::getHelp() {
    GtkWidget *vbox;
    GtkWidget *label;
    GdkColor color;
    std::stringstream sstr;
    
    color.red = 0xFFFF;
    color.green = 0xFFFF;
    color.blue = 0xFFFF;
    
    vbox = gtk_vbox_new(FALSE, 0);
    
    sstr << "Dart Cricket 10 is a game just like\n"
                    << "regular Cricket, except it ends after round 10.\n"
                    << "The winner is the person who has the most points\n"
                    << "at the end of the game.  If there is a tie in \n"
                    << "points, the winner is then the person who has the\n"
                    << "most marks.\n"
                    << "If a player closes out the board and has the most\n"
                    << "points, the game is over and that player is the winner.\n";
    
    label = gtk_label_new(sstr.str().c_str());
    gtk_widget_modify_fg(label, (GtkStateType)0, &color);
    gtk_box_pack_start((GtkBox *)vbox, label, FALSE, FALSE, 10);
    return vbox;
}

int DartCricket10::gameType() const {
	return DART_GAME;
}

int DartCricket10::getWinner() {
	for (int i = 0; i < numberOfPlayers; ++i) {
		finalScore[i] = atoi(gtk_label_get_text((GtkLabel *)playerScoreLabels[i]));
	}
	if (winner == -1) { // we have to see who won here
		int topscore[8];
		int totalTopScore(0);
		int topmarks[8];
		int totalTopMarks(0);
		bool scoreTie(false);
		for (int i = 0; i < 8; ++i) {
			topscore[i] = 0;
			topmarks[i] = 0;
		}
		for (int i = 0; i < numberOfPlayers; ++i) {
			if (totalTopScore < finalScore[i]) {
				topscore[i] = 1;
				scoreTie = false;
				totalTopScore = finalScore[i];
				winner = i;
			}
			else if (totalTopScore == finalScore[i]) {
				scoreTie = true;
			}
			else {
				topscore[i] = 0;
			}
		}
		if (scoreTie) { // no winner yet
			// clear the players who don't have this top score
			for (int i = 0; i < numberOfPlayers; ++i) {
				if (finalScore[i] == totalTopScore) {
					topscore[i] = 1;
				}
				else {
					topscore[i] = 0;
				}
			}
			scoreTie = 0;
			// now do the same for topmarks
			for (int i = 0; i < numberOfPlayers; ++i) {
				if (totalTopMarks < getMarks(i) && finalScore[i] == totalTopScore) {
					topmarks[i] = 1;
					scoreTie = false;
					totalTopMarks = getMarks(i);
					winner = i;
				}
				else if (totalTopMarks == getMarks(i) && finalScore[i] == totalTopScore) {
					scoreTie = true;
				}
				else {
					topmarks[i] = 0;
				}
			}
			if (scoreTie) {
				// must do something better.
				return -1; // no winner
			}
			else {
				for (int i = 0; i < numberOfPlayers; ++i) {
					if (finalScore[i] == totalTopScore && getMarks(i) == totalTopMarks) {
						winner = i;
						return i;
					}
				}
			}
		}
		else {	// no tie, so this player wins
			for (int i = 0; i < numberOfPlayers; ++i) {
				if (finalScore[i] == totalTopScore) {
					winner = i;
					return i;
				}
			}
		}
	}
 return -1;
}

int DartCricket10::getMarks(int player) {
	int marks(0);
	for (int i = 0; i < 7; ++i) {
		marks += closeMarks[player][i];
	}
	return marks;
}
